#include "project.h"


#ifdef USE_RINGS
void check_rings()
{
 int i = 0;

 for(i=0; i<MAX_RINGS; i++) {
     if(rings[i].active && !rings[i].delay) {
         if(rings[i].r_change) {
             rings[i].radius += rings[i].r_change;
             if(rings[i].radius <= 0.00) { init_ring(i); }
                               }

         if(rings[i].a_change) {
             rings[i].alpha += rings[i].a_change;
             if(rings[i].alpha >  1.00) { rings[i].alpha = 1.00; }
             if(rings[i].alpha <= 0.00) { init_ring(i);          }
                               }
                                            }

     // Rings can be spawned with an initial delay, preventing drawing
     // or logical changes. Use this to create concentric rings with one group of spawn calls
     //
     if(rings[i].delay) {
         rings[i].delay--;
         if(rings[i].delay < 0) { rings[i].delay = 0; }
                        }
                            }

 return;
}




// NOTE: No implicit sorting or merging. Do that yourself if you want
//
void draw_rings()
{
 int i = 0;


 for(i=0; i<MAX_RINGS; i++) {
     if(rings[i].active && !rings[i].delay) {
         DrawRing(rings[i].pos, rings[i].radius, rings[i].radius + rings[i].thickness, 0.0, 360.00, rings[i].segments, Fade(rings[i].col, rings[i].alpha));
                                            }
                            }

 return;
}




// NOTE: Clears out a ring. Use when initializing game vars
//
void init_ring(int slot)
{
 if(slot < 0 || slot >= MAX_RINGS) { return; } // report error to framework

 rings[slot].pos.x     = 0.0;
 rings[slot].pos.y     = 0.0;
 rings[slot].radius    = 0.0;
 rings[slot].r_change  = 0.0;
 rings[slot].alpha     = 1.0;
 rings[slot].a_change  = 0.0;
 rings[slot].thickness = 1.0;
 rings[slot].segments  = 32;
 rings[slot].delay     = 0;
 rings[slot].active    = 0;

 return;
}




// NOTE: Locates an inactive ring. Returns -1 on failure
//
int get_free_ring()
{
 int i = 0;

 for(i=0; i<MAX_RINGS; i++) {
     if(!rings[i].active) { return(i); }
                            }


 return(-1);
}




int spawn_ring(float x, float y, float radius, float thickness, int delay, Color col)
{

 int slot = get_free_ring();
 if(slot == -1) { return(-1); }  // Normal, non-critical failure (no resources available)

 rings[slot].pos.x     = x;
 rings[slot].pos.y     = y;
 rings[slot].radius    = radius;
 rings[slot].r_change  = 0.0;
 rings[slot].alpha     = 1.0;
 rings[slot].a_change  = 0.0;
 rings[slot].thickness = thickness;
 rings[slot].segments  = 32;
 rings[slot].delay     = delay;
 rings[slot].col       = col;
 rings[slot].active    = 1;

 return(slot);
}

#endif
