#include "Project.h"

#ifdef USE_RTMIDI



void init_midi()
{
 int i = 0;

  try {
    midi.midiin = new RtMidiIn();
  }
  catch ( RtMidiError &error ) {
    error.printMessage();
    exit( EXIT_FAILURE );
  }

 midi.total_ports  = midi.midiin->getPortCount();
 midi.current_port = 0;

 midi.ignore_sysex = 0;
 midi.ignore_timing = 0;
 midi.ignore_active_sensing = 0;

 if(midi.total_ports > 0) {
 midi.midiin->openPort(midi.current_port);

 midi.midiin->ignoreTypes(midi.ignore_sysex, midi.ignore_timing, midi.ignore_active_sensing);

 //log("[ MIDI ] : Ports found = %d", midi.total_ports);
 //log("[ MIDI ] : Reading from source = %s", midi.midiin->getPortName().c_str()); // I'm actually a little surprised that worked
                          }                                                      // Printf can't handle std::strings. Hence, the .c_str()

 else { /*log("[ INIT ] : No MIDI input detected");*/ }


 // Initialize devices with safe values
 //
 for(i=0; i<MAX_KNOBS; i++) {
     midi.knobs[i].id = -1;
     rescale_midi_device_abs(midi.knobs[i], 0, 127, 0, 127);
     rescale_midi_device_abs(midi.pads[i], 0, 127, 0, 127);
                            }

 return;
}






void poll_midi()
{
 int i            = 0;
 int _key         = 0;
 int recheck      = 0;
 double freq_calc = 0.0;

 if(midi.total_ports < 1) { return; } // Nothing to poll

 repoll:

 midi.msg_stamp = midi.midiin->getMessage(&midi.message);
 midi.msg_bytes = midi.message.size();

 if(midi.msg_bytes > 0) {

     // Knob or Pad update
     if(midi.message[0] == 176) {
         for(i=0; i<MAX_KNOBS; i++) {
             if(midi.knobs[i].id == midi.message[1]) {
                 midi.knobs[i].input = midi.message[2];
                 update_midi_knob(i);
                                                     }
                                    }
                                }

     // Key press
     if(midi.message[0] == 144) {
         _key = midi.message[1]; // sanitize this, yo
         freq_calc = (440.0 / 32.0) * pow(2.0, ((double)(_key - 9) / 12.0));
         midi.keys[_key] = 1;

         midi.note_key = _key;
         midi.note_freq = freq_calc;
         midi.note_active = 1;
                                }

     // Key release
     if(midi.message[0] == 128) {
         _key = midi.message[1]; // sanitize this, yo
         midi.keys[_key] = 0;
         midi.note_key = 0;
         midi.note_active = 0;
                                }

     goto repoll;
                        }


 return;
}




// NOTE: For absolute scale devices
//
void rescale_midi_device_abs(device_struct &knob, float raw_min, float raw_max, float scale_min, float scale_max)
{
 knob.input = 0;
 knob.value = scale_min;

 knob.mode  = 0;
 knob.r_min = raw_min;
 knob.r_max = raw_max;
 knob.s_min = scale_min;
 knob.s_max = scale_max;
 knob.scale = 1.00;          // Not used

 return;
}




// NOTE: For relative devices
//
void rescale_midi_device_rel(device_struct &knob, float scale_min, float scale_max, float tick_scale)
{
 knob.input = 0;
 //knob.value = scale_min;

 knob.mode  = 1;
 knob.r_min = 0.0;           // Not used
 knob.r_max = 1.0;           // Not used
 knob.s_min = scale_min;
 knob.s_max = scale_max;
 knob.scale = tick_scale;

 return;
}




void update_midi_knob(int slot)
{
 int i = slot;

     if(!midi.knobs[i].mode) { // Absolute Values
         midi.knobs[i].value = scale(midi.knobs[i].input, midi.knobs[i].r_min, midi.knobs[i].r_max, midi.knobs[i].s_min, midi.knobs[i].s_max);
                             }

     else { // Relative Values
         midi.knobs[i].value += ((float)(midi.knobs[i].input - 64) * midi.knobs[i].scale);
         midi.knobs[i].input = 0;
         if(midi.knobs[i].value < midi.knobs[i].s_min) { midi.knobs[i].value = midi.knobs[i].s_min; }
         if(midi.knobs[i].value > midi.knobs[i].s_max) { midi.knobs[i].value = midi.knobs[i].s_max; }
          }

     // midi.pads[i].value  = scale(midi.pads[i].input, midi.pads[i].r_min, midi.pads[i].r_max, midi.pads[i].s_min, midi.pads[i].s_max);


 return;
}




void draw_knob(float x, float y, float angle, int slot)
{
 Vector2 origin    = { tex_knob.width, tex_knob.height };
 Vector2 len       = { 0 };
 float final_angle = 0.0;

 // Rotate knob to correct angle based off value and scale
 //
 final_angle = -120 + scale(angle, midi.knobs[slot].s_min, midi.knobs[slot].s_max, 0, 243);
 DrawTexturePro(tex_knob, (Rectangle){0, 0, tex_knob.width, tex_knob.height}, (Rectangle){x, y, tex_knob.width * 2, tex_knob.height * 2} , origin, final_angle, WHITE);


 // Draw Actual value above knob
 //
 sprintf(custom_text, "%02.02f", angle);
 len = MeasureTextEx(font3, custom_text, font3.baseSize * 3, 1);
 DrawTextEx(font3, custom_text, (Vector2){ x - (len.x / 2.0), y + 30 }, font3.baseSize * 3, 1, BLACK);

 return;
}





void draw_midi_knobs()
{
 int i = 0;

 DrawRectangle(0, SCREEN_H, SCREEN_W, 300, GRAY);

 // Knobs on Arturia start at 1. Sigh.
 //
 for(i=0; i<8; i++) {
     DrawTextureEx(tex_knob_plate, (Vector2){140 * i, SCREEN_H + 20}, 0, 2.0, WHITE);
     draw_knob((140 * i) + 64, (SCREEN_H + 20) + 68, midi.knobs[i+1].value, i+1);
                    }

 for(i=0; i<8; i++) {
     DrawTextureEx(tex_knob_plate, (Vector2){140 * i, SCREEN_H + 160}, 0, 2.0, WHITE);
     draw_knob((140 * i) + 64, (SCREEN_H + 160) + 68, midi.knobs[i+9].value, i+9);
                    }

 return;
}





void set_knob(int slot, float new_value)
{

 midi.knobs[slot].value = new_value;


 return;
}





// NOTE: Sets up interface for my current Arturia Minilab II profile
//
void init_arturia_profile()
{
 int i = 0;
 int ids[17] = { -1, 112, 74, 71, 76, 77, 93, 73, 75, 114, 18, 19, 16, 17, 91, 79, 72 };

 if(MAX_KNOBS < 17) { exit(0); }

 for(i=0; i<17; i++) {
     rescale_midi_device_rel(midi.knobs[i], 0.0, 100.0, 1.0);
     midi.knobs[i].id = ids[i];
                     }

 rescale_midi_device_rel(midi.knobs[10], 0.0, 360.0, 2.0);
 rescale_midi_device_rel(midi.knobs[11], 0.0, 1.0, 0.01);
 rescale_midi_device_rel(midi.knobs[12], 0.0, 1.0, 0.01);

 rescale_midi_device_rel(midi.knobs[14], 0.0, 360.0, 2.0);
 rescale_midi_device_rel(midi.knobs[15], 0.0, 1.0, 0.01);
 rescale_midi_device_rel(midi.knobs[16], 0.0, 1.0, 0.01);


 rescale_midi_device_rel(midi.knobs[2], 0.0, 360.0, 2.0);
 rescale_midi_device_rel(midi.knobs[3], 0.0, 1.0, 0.01);
 rescale_midi_device_rel(midi.knobs[4], 0.0, 1.0, 0.01);

 rescale_midi_device_rel(midi.knobs[6], 0.0, 360.0, 2.0);
 rescale_midi_device_rel(midi.knobs[7], 0.0, 1.0, 0.01);
 rescale_midi_device_rel(midi.knobs[8], 0.0, 1.0, 0.01);

 return;
}




void save_knobs()
{
 FILE *file;
 int i     = 0;
 int b_val = 0;

 file = fopen("knobs.sav", "wb");

 for(i=0; i<MAX_KNOBS; i++) {
     b_val = (int)midi.knobs[i].mode;
     fwrite((void *)&b_val, 1, sizeof(b_val), file);
     fwrite((void *)&midi.knobs[i].id,    1, sizeof(midi.knobs[i].id),    file);
     fwrite((void *)&midi.knobs[i].r_min, 1, sizeof(midi.knobs[i].r_min), file);
     fwrite((void *)&midi.knobs[i].r_max, 1, sizeof(midi.knobs[i].r_max), file);
     fwrite((void *)&midi.knobs[i].s_min, 1, sizeof(midi.knobs[i].s_min), file);
     fwrite((void *)&midi.knobs[i].s_max, 1, sizeof(midi.knobs[i].s_max), file);
     fwrite((void *)&midi.knobs[i].value, 1, sizeof(midi.knobs[i].value), file);
     fwrite((void *)&midi.knobs[i].scale, 1, sizeof(midi.knobs[i].scale), file);
                            }

 fclose(file);

 return;
}




void load_knobs()
{
 FILE *file;
 int i     = 0;
 int b_val = 0;

 file = fopen("knobs.sav", "rb");

 for(i=0; i<MAX_KNOBS; i++) {
     fread((void *)&b_val, 1, sizeof(b_val), file);
     midi.knobs[i].mode = b_val;

     fread((void *)&midi.knobs[i].id,    1, sizeof(midi.knobs[i].id),    file);
     fread((void *)&midi.knobs[i].r_min, 1, sizeof(midi.knobs[i].r_min), file);
     fread((void *)&midi.knobs[i].r_max, 1, sizeof(midi.knobs[i].r_max), file);
     fread((void *)&midi.knobs[i].s_min, 1, sizeof(midi.knobs[i].s_min), file);
     fread((void *)&midi.knobs[i].s_max, 1, sizeof(midi.knobs[i].s_max), file);
     fread((void *)&midi.knobs[i].value, 1, sizeof(midi.knobs[i].value), file);
     fread((void *)&midi.knobs[i].scale, 1, sizeof(midi.knobs[i].scale), file);
                            }

 fclose(file);

 return;
}

#endif
