#include "project.h"





// NOTE: Used to initially populate important stuff that'll never change between sessions
//       Puppet is the player actor, and so its virtual gamepad will be tracked for input.
//
void spawn_permanent_actors()
{
 int who = -1;

 // Create player
 //
 puppet = spawn_player(200, 600, 0);
 if(puppet == -1) { log("Fatal error. Couldn't spawn player. Baborting."); exit(0); }

 who = spawn_rocket_flame(puppet);
 if(who != -1) {
     actors[puppet].extra[0] = who;
     actors[who].cel_x     = 2;
     actors[who].color     = ORANGE;
     actors[who].state     = 0;
     actors[who].following = puppet;
     actors[who].follow_x  = -96;
     actors[who].follow_y  = 0;
               }
 else { actors[puppet].extra[0] = 0; }

 return;
}





// Spawns a new actor. Types are 0-x for default skin
//
int spawn_actor(int type, float x, float y, float z, int cell_x, float path_x, float path_y, float path_z, bool force_reverse)
{
 int slot = -1;

 if(!force_reverse) { slot = get_free_actor(0); }
 else               { slot = get_free_actor(1); }


 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { return(-1); }

 actors[slot].mode      = 0;
 actors[slot].visible   = 1;
 actors[slot].effect    = 0;
 actors[slot].projected = 0;
 actors[slot].blend     = 1.0f;
 actors[slot].angle     = 0.0f;
 actors[slot].timer     = 0;
 actors[slot].stamina   = 10000;
 actors[slot].v_angle   = 0.0;
 actors[slot].locked_to = -1;

 actors[slot].pos.x     = x;
 actors[slot].pos.y     = y;
 actors[slot].pos.z     = z;

 actors[slot].path.x    = path_x;
 actors[slot].path.y    = path_y;
 actors[slot].path.z    = path_z;

 actors[slot].cel       = tex_32x; // default value
 actors[slot].cel_x     = cell_x;
 actors[slot].cel_y     = 0;

 actors[slot].state     = 0;
 actors[slot].scale     = 1.0;

 actors[slot].enabled = 1;

 return(slot);
}





int spawn_player(int x, int y, int z)
{
 int who = -1;

 who = spawn_actor(0, 0, 0, 0, 1, 0.0, 0.0, 0.0, 0);

 if(who != -1) {
     actors[who].angle   = 0;
     actors[who].team    = 1;
     actors[who].scale   = 3.0;
     actors[who].state   = 0;
     actors[who].pos.x   = x;
     actors[who].pos.y   = y;
     actors[who].pos.z   = z;
     actors[who].cel     = tex_32x;
     actors[who]._size_w = 32;
     actors[who]._size_h = 32;
     actors[who].stamina = 1000;
     actors[who].color   = WHITE;
     actors[who].enabled = 1;
               }

 return(who);
}





void kill_actor(int slot)
{

 // Basic Sanity Check
 //
 if(slot < 0 || slot >= MAX_ACTORS) { log("ERROR: Invalid slot in kill command. Slot = %d", slot); return; }

 int i = 0;

 // If an actor is following us, standard behavior would be to kill it
 // Leave a state open for this (99)
 //
 for(i=0; i<MAX_ACTORS; i++) {
     if(actors[i].enabled && actors[i].following == slot && i != slot) { actors[i].state = 99; }
                             }
 /*
 // Check if tip is active
 //
 if(actors[slot].tip != -1) {
     if(actors[slot].tip >= 0 && actors[slot].tip < MAX_TIPS) {
         if(textbox[actors[slot].tip].hold) { textbox[actors[slot].tip].hold = 0; }
                                                              }
                            }
 */

 actors[slot].enabled     = 0;
 actors[slot].immortal    = 0;
 actors[slot].visible     = 1;
 actors[slot].projected   = 0;
 actors[slot].mirror      = 0;
 actors[slot].team        = 0;
 actors[slot].decal       = 0;
 actors[slot].shader      = 0;
 actors[slot].shadow      = 0;
 actors[slot].accessory   = 0;
 actors[slot].y_shift     = 0;
 actors[slot].jitter_x    = 0;
 actors[slot].jitter_y    = 0;
 actors[slot].slew        = 0;
 actors[slot].gravity     = 0;
 actors[slot].launched    = 0;
 actors[slot].flicker     = 0;
 actors[slot].anim_set    = 0;
 actors[slot].anim_step   = 0;
 actors[slot].anim_timing = 0;
 actors[slot].anim_loop   = 0;
 actors[slot].effect      = 0;
 actors[slot].state       = 0;
 actors[slot].mode        = 0;
 actors[slot].flip        = 0;
 actors[slot].timer       = 0;
 actors[slot]._size_w     = 16; // Default value
 actors[slot]._size_h     = 16; // Default value
 actors[slot].inert       = 0;
 actors[slot].extra[0]    = 0;
 actors[slot].extra[1]    = 0;
 actors[slot].extra[2]    = 0;
 actors[slot].extra[3]    = 0;
 actors[slot].home[0]     = -1;
 actors[slot].home[1]     = -1;
 actors[slot].home[2]     = -1;
 actors[slot].preference  = -1;
 actors[slot].pref_delay  = 0;
 actors[slot].event       = 0;
 actors[slot].timer       = 0;
 actors[slot].cel_x       = 0;
 actors[slot].cel_y       = 0;
 actors[slot].tip         = -1;
 actors[slot].following   = -1;
 actors[slot].follow_x    = 0;
 actors[slot].follow_y    = 0;
 actors[slot].owner       = -1;
 actors[slot].locked_to   = -1;
 actors[slot].scale       = 1.0f;
 actors[slot].v_angle     = 0.0f;
 actors[slot].v_roll      = 0.0f;
 actors[slot].v_speed     = 0.0f;
 actors[slot].blend       = 1.0f;
 actors[slot].angle       = 0.0f;
 actors[slot].roll        = 0.0f;
 actors[slot].quake       = 0.0f;
 actors[slot].angle       = 0.0f;
 actors[slot].last_angle  = 0.0f;
 actors[slot].speed       = 1.0f; // Default value
 actors[slot].stamina     = 1000;

 actors[slot].updates     = 1;
 actors[slot].moved_up    = 0;
 actors[slot].moved_down  = 0;
 actors[slot].moved_wait  = 0;

 for(i=0; i<4; i++) {
     actors[slot].d_pad[i]    = 0;
     actors[slot].d_repeat[i] = 0;
     actors[slot].buttons[i]  = 0;
     actors[slot].b_repeat[i] = 0;
                    }

 actors[slot].shader      = 0;
 for(i=0; i<10; i++) { actors[slot].s_vars[i] = 0.0; }

 actors[slot].path.x      = 0;
 actors[slot].path.y      = 0;
 actors[slot].path.z      = 0;

 actors[slot].pos.x       = 0;
 actors[slot].pos.y       = 0;
 actors[slot].pos.z       = 0;

 actors[slot].dest.x      = 0;
 actors[slot].dest.y      = 0;
 actors[slot].dest.z      = 0;

 actors[slot].offset.x    = 0;
 actors[slot].offset.y    = 0;
 actors[slot].offset.z    = 0;

 actors[slot].cel         = tex_32x;
 actors[slot].color       = WHITE;
 actors[slot].ref_color   = WHITE;


 return;
}
